<?php

namespace Memsource\MemsourceConnector\Controller;

use Memsource\MemsourceConnector\Domain\Model\ConfigRepository;
use Memsource\MemsourceConnector\Domain\Service\Logger;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Object\ObjectManager;

class EidController extends ActionController
{
    const CODE_OK = 200;
    const CODE_BAD_REQUEST = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SERVER_ERROR = 500;

    const STATUS_ERROR = ['status' => 'ERROR'];
    const STATUS_OK = ['status' => 'OK'];

    /**
     * Process HTTP request.
     */
    public function run()
    {
        $token = GeneralUtility::_GET('token');
        $resource = GeneralUtility::_GET('resource');
        $id = GeneralUtility::_GET('id');
        $parent = GeneralUtility::_GET('parent');
        $targetLang = GeneralUtility::_GET('target_language');
        $translatedTitle = GeneralUtility::_POST('title');
        $translatedContent = GeneralUtility::_POST('content');

        if (!$this->validateToken($token)) {
            $this->sendResponse(['message' => 'Invalid token provided.'], self::CODE_UNAUTHORIZED);
        }

        $controller = $this->getController($resource);

        if ($controller === null) {
            $this->sendResponse(
                ['message' => "Unsupported resource type '" . ($resource ?: 'NULL') . "'."],
                self::CODE_BAD_REQUEST
            );
        }

        $method = $this->getHttpMethod();

        if ($method === null) {
            $this->sendResponse(['message' => 'Unable to detect HTTP method.'], self::CODE_SERVER_ERROR);
        }

        $action = $this->getAction($method, $id);

        if ($action === null || !method_exists($controller, $action)) {
            $controllerName = basename(str_replace('\\', '/', get_class($controller)));
            $this->sendResponse(
                ['message' => "Unsupported action '" . ($action ?: 'NULL') . "' for controller '$controllerName'."],
                self::CODE_BAD_REQUEST
            );
        }

        try {
            $result = $controller->$action($id ?: $parent, $targetLang, $translatedTitle, $translatedContent);
            $this->sendResponse($result);
        } catch (\Exception $e) {
            $this->sendResponse(['message' => 'Request failed: ' . $e->getMessage()], self::CODE_SERVER_ERROR, $e);
        }
    }

    /**
     * Check token before performing the request.
     *
     * @param string $token
     * @return bool
     */
    private function validateToken($token): bool
    {
        $storedToken = GeneralUtility::makeInstance(ConnectionPool::class)
            ->getConnectionForTable('tx_memsourceconnector_domain_model_config')
            ->select(
                ['token'],
                'tx_memsourceconnector_domain_model_config',
                ['key' => ConfigRepository::CONFIG_KEY]
            )
            ->fetchColumn();

        return $storedToken === $token;
    }

    /**
     * Get current HTTP method.
     *
     * @return string|null
     */
    private function getHttpMethod()
    {
        if (isset($GLOBALS['TYPO3_REQUEST']) && method_exists($GLOBALS['TYPO3_REQUEST'], 'getMethod')) {
            return $GLOBALS['TYPO3_REQUEST']->getMethod();
        }

        if (isset($_SERVER['REQUEST_METHOD'])) {
            return $_SERVER['REQUEST_METHOD'];
        }

        return null;
    }

    /**
     * Get instancion of a controller according to the current request.
     *
     * @param string $resource
     * @return object|null
     */
    private function getController($resource)
    {
        $objectManager = GeneralUtility::makeInstance(ObjectManager::class);

        $controllerName = 'Memsource\\MemsourceConnector\\Controller\\' . ucfirst(trim($resource)) . 'Controller';

        if ($resource === null || !class_exists($controllerName)) {
            return null;
        }

        return $objectManager->get($controllerName);
    }

    /**
     * Get controller's action according to the current request.
     *
     * @param string $method
     * @param int $id
     * @return string|null
     */
    private function getAction($method, $id)
    {
        if ($method === 'POST' && $id !== null) {
            return 'storeTranslationAction';
        }

        if ($method === 'GET' && $id !== null) {
            return 'getDetailAction';
        }

        if ($method === 'GET') {
            return 'getListAction';
        }

        return null;
    }

    /**
     * Print JSON response and exit.
     *
     * @param array $body
     * @param int $code
     * @param \Exception|null $exception
     */
    private function sendResponse($body, $code = self::CODE_OK, $exception = null)
    {
        header('Content-Type: application/json');

        if ($code === self::CODE_BAD_REQUEST) {
            header('HTTP/1.0 400 Bad Request');
        } elseif ($code === self::CODE_UNAUTHORIZED) {
            header('HTTP/1.0 401 Unauthorized');
        } elseif ($code === self::CODE_SERVER_ERROR) {
            header('HTTP/1.1 500 Internal Server Error');
        } elseif ($code !== self::CODE_OK) {
            header('HTTP/1.1 500 Internal Server Error');
            $body = ['message' => "Unsupported HTTP status code provided: $code"];
        }

        if ($code === self::CODE_OK) {
            $body = self::STATUS_OK + $body;
        } else {
            $body = self::STATUS_ERROR + $body;
        }

        if ($this->isDebugEnabled()) {
            $objectManager = GeneralUtility::makeInstance(ObjectManager::class);
            $logger = $objectManager->get(Logger::class);
            $logger->logHttpRequest($this->getHttpMethod(), $this->getRequestUrl(), $_POST, $code, $body, $exception);
        }

        exit(json_encode($body));
    }

    /**
     * Check if debug mode is active.
     *
     * @return bool
     */
    private function isDebugEnabled(): bool
    {
        $debug = GeneralUtility::makeInstance(ConnectionPool::class)
            ->getConnectionForTable('tx_memsourceconnector_domain_model_config')
            ->select(
                ['debug'],
                'tx_memsourceconnector_domain_model_config',
                ['key' => ConfigRepository::CONFIG_KEY]
            )
            ->fetchColumn();

        return $debug === 1;
    }

    /**
     * Get URL of current request.
     *
     * @return string
     */
    private function getRequestUrl(): string
    {
        if (
            isset($GLOBALS['TYPO3_REQUEST']) &&
            isset($GLOBALS['TYPO3_REQUEST']->getAttributes()['normalizedParams']) &&
            $GLOBALS['TYPO3_REQUEST']->getAttributes()['normalizedParams']->getRequestUrl()
        ) {
            return $GLOBALS['TYPO3_REQUEST']->getAttributes()['normalizedParams']->getRequestUrl();
        }

        if (isset($_SERVER['REQUEST_URI'])) {
            return $_SERVER['REQUEST_URI'];
        }

        if (isset($_SERVER['QUERY_STRING'])) {
            return $_SERVER['QUERY_STRING'];
        }

        return '';
    }
}
