<?php

namespace Memsource\MemsourceConnector\Controller;

use Memsource\MemsourceConnector\Domain\Model\AccordionRepository;
use Memsource\MemsourceConnector\Domain\Model\CarouselRepository;
use Memsource\MemsourceConnector\Domain\Model\ContentRepository;
use Memsource\MemsourceConnector\Domain\Model\ImageRepository;
use Memsource\MemsourceConnector\Domain\Model\LanguageRepository;
use Memsource\MemsourceConnector\Domain\Model\NewsRepository;
use Memsource\MemsourceConnector\Domain\Model\PageRepository;
use Memsource\MemsourceConnector\Domain\Model\TabRepository;
use Memsource\MemsourceConnector\Domain\Service\Serializer;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;

class PagesController extends ActionController
{
    /**
     * Get list of pages.
     *
     * @param int|null $parent Parent page ID.
     * @return array
     */
    public function getListAction($parent): array
    {
        $pageRepository = GeneralUtility::makeInstance(PageRepository::class);
        $newsRepository = GeneralUtility::makeInstance(NewsRepository::class);

        $pages = $pageRepository->findByParent($parent);
        $news = $newsRepository->findByParent($parent);

        return [
            'pages' => array_merge($pages, $news)
        ];
    }

    /**
     * Get page by ID.
     *
     * @param int $id Page ID.
     * @param int|null $languageId Language ID.
     * @return array
     */
    public function getDetailAction($id, $languageId): array
    {
        $contentRepository = GeneralUtility::makeInstance(ContentRepository::class);
        $serializer = GeneralUtility::makeInstance(Serializer::class);

        if (strpos($id, NewsRepository::ID_PREFIX) !== false) {
            $repository = GeneralUtility::makeInstance(NewsRepository::class);
            $uid = str_replace(NewsRepository::ID_PREFIX, '', $id);
            $relField = 'tx_news_related_news';
        } else {
            $repository = GeneralUtility::makeInstance(PageRepository::class);
            $uid = $id;
            $relField = 'pid';
        }

        $page = $repository->findOneByUid((int) $uid, $languageId);

        if ($page->isTranslation()) {
            $where = [
                'pid' => $page->getL10nParent(),
                'sys_language_uid' => $page->getSysLanguageUid()
            ];
        } else {
            $where = [
                $relField => $uid,
                'sys_language_uid' => 0
            ];
        }

        $content = $contentRepository->findBy($where);

        $serialized = $serializer->serializePage($page, $content);

        if (isset($_GET['raw']) && $_GET['raw'] === '1') {
            echo "==> title: " . $page->getTitle() . "\n==> body:\n$serialized\n===\n\n";
        }

        return [
            'page' => [
                'id' => $id,
                'title' => $page->getTitle(),
                'body' => $serialized
            ]
        ];
    }

    /**
     * Store translated page.
     *
     * @param int|string $sourcePageId
     * @param string $targetLangCode
     * @param string $title
     * @param string $body
     * @return array
     */
    public function storeTranslationAction($sourcePageId, $targetLangCode, $title, $body): array
    {
        $languageRepository = GeneralUtility::makeInstance(LanguageRepository::class);
        $contentRepository = GeneralUtility::makeInstance(ContentRepository::class);
        $imageRepository = GeneralUtility::makeInstance(ImageRepository::class);
        $carouselRepository = GeneralUtility::makeInstance(CarouselRepository::class);
        $accordionRepository = GeneralUtility::makeInstance(AccordionRepository::class);
        $tabRepository = GeneralUtility::makeInstance(TabRepository::class);
        $serializer = GeneralUtility::makeInstance(Serializer::class);
        $detailIdPrefix = '';

        if (strpos($sourcePageId, NewsRepository::ID_PREFIX) !== false) {
            $pageRepository = GeneralUtility::makeInstance(NewsRepository::class);
            $sourcePageId = str_replace(NewsRepository::ID_PREFIX, '', $sourcePageId);
            $detailIdPrefix = NewsRepository::ID_PREFIX;
        } else {
            $pageRepository = GeneralUtility::makeInstance(PageRepository::class);
        }

        $sourcePageId = (int) $sourcePageId;
        $page = $serializer->deserializePage($body);
        $targetLangId = $languageRepository->getLangIdByCode($targetLangCode, $sourcePageId);
        $targetPageId = $pageRepository->getOrCreateTargetPage($sourcePageId, $targetLangId);

        $pageRepository->update($targetPageId, $title, $page);

        foreach ($page['content'] as $sourceContentId => $translatedContent) {
            $targetContentId = $contentRepository->getOrCreateTargetContent($sourcePageId, $sourceContentId, $targetLangId);
            $contentRepository->update(
                $targetContentId,
                $translatedContent['header'],
                $translatedContent['subheader'],
                $translatedContent['bodytext']
            );
            $imageRepository->bulkUpdate($translatedContent['images'], $sourcePageId, $targetContentId, $targetLangId, ContentRepository::TABLE_NAME);

            foreach ($translatedContent['carousel'] as $carouselId => $carousel) {
                $targetCarouselId = $carouselRepository->getOrCreateTarget($sourcePageId, $targetContentId, $carouselId, $targetLangId);
                $carouselRepository->update($targetCarouselId, $carousel['header'], $carousel['subheader'], $carousel['bodytext']);
                $imageRepository->bulkUpdate($carousel['images'], $sourcePageId, $targetCarouselId, $targetLangId, CarouselRepository::TABLE_NAME);
            }

            foreach ($translatedContent['accordion'] as $accordionlId => $accordion) {
                $targetAccordionId = $accordionRepository->getOrCreateTarget($sourcePageId, $targetContentId, $accordionlId, $targetLangId);
                $accordionRepository->update($targetAccordionId, $accordion['header'], $accordion['bodytext']);
                $imageRepository->bulkUpdate($accordion['images'], $sourcePageId, $targetAccordionId, $targetLangId, AccordionRepository::TABLE_NAME);
            }

            foreach ($translatedContent['tab'] as $tabId => $tab) {
                $targetTabId = $tabRepository->getOrCreateTarget($sourcePageId, $targetContentId, $tabId, $targetLangId);
                $tabRepository->update($targetTabId, $tab['header'], $tab['bodytext']);
                $imageRepository->bulkUpdate($tab['images'], $sourcePageId, $targetTabId, $targetLangId, TabRepository::TABLE_NAME);
            }
        }

        $imageRepository->bulkUpdate($page['images'], $sourcePageId, $targetPageId, $targetLangId, PageRepository::getTableName());

        return $this->getDetailAction($detailIdPrefix . $targetPageId, $targetLangId);
    }
}
