<?php

namespace Memsource\MemsourceConnector\Domain\Model;

use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Persistence\Repository;

class ConfigRepository extends Repository
{
    const TABLE = 'tx_memsourceconnector_domain_model_config';
    const CONFIG_KEY = 'memsource-config';
    const TOKEN_LENGTH = 32;
    const DEBUG_DEFAULT = false;

    /**
     * Get Memsource Connector config (access token and debug mode).
     *
     * @return Config
     */
    public function getConfig(): Config
    {
        $config = $this->findConfig();

        if ($config === null) {
            $config = $this->createNewConfig();
        }

        return $config;
    }

    /**
     * Is debug mode active?
     *
     * @return bool
     */
    public function isDebugEnabled(): bool
    {
        $config = $this->findConfig();

        return ($config !== null) && $config->isDebug();
    }

    /**
     * Create and store new token.
     */
    public function regenerateToken()
    {
        $config = $this->findConfig();
        $config->setToken($this->generateRandomToken());
        $this->persistenceManager->update($config);
    }

    /**
     * Disable debug mode.
     */
    public function disableDebug()
    {
        $config = $this->findConfig();
        $config->setDebug(false);
        $this->persistenceManager->update($config);
    }

    /**
     * Enable debug mode.
     */
    public function enableDebug()
    {
        $config = $this->findConfig();
        $config->setDebug(true);
        $this->persistenceManager->update($config);
    }

    /**
     * Validate provided access token.
     *
     * @param string $token
     * @return bool
     */
    public function validateToken($token): bool
    {
        return $this->findConfig()->getToken() === $token;
    }

    /**
     * Find current configuration in database.
     *
     * @return Config
     */
    private function findConfig()
    {
        return $this->findOneByKey(self::CONFIG_KEY);
    }

    /**
     * Create new configuration with generated token.
     *
     * @return Config
     */
    private function createNewConfig(): Config
    {
        $this->cleanupConfig();

        $config = new Config(
            self::CONFIG_KEY,
            $this->generateRandomToken(),
            self::DEBUG_DEFAULT
        );

        $this->add($config);
        $this->persistenceManager->persistAll();

        return $config;
    }

    /**
     * Remove all existing configurations from DB.
     */
    private function cleanupConfig()
    {
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
            ->getQueryBuilderForTable(self::TABLE);

        $queryBuilder
            ->getRestrictions()
            ->removeAll();

        $queryBuilder
            ->delete(self::TABLE)
            ->where("`key` = '" . self::CONFIG_KEY . "'")
            ->execute();
    }

    /**
     * Generate random string.
     *
     * @return string
     */
    private function generateRandomToken(): string
    {
        $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        $token = '';
        $max = strlen($characters) - 1;

        for ($i = 0; $i < self::TOKEN_LENGTH; $i++) {
            $token .= $characters[random_int(0, $max)];
        }

        return $token;
    }
}
