<?php

namespace Memsource\MemsourceConnector\Domain\Model;

use Memsource\MemsourceConnector\Exception\NotFoundException;
use TYPO3\CMS\Core\Utility\GeneralUtility;

class ContentRepository
{
    const TABLE_NAME = 'tt_content';

    /**
     * Find content by given restriction.
     *
     * @param array $where
     * @return array
     * @throws NotFoundException
     */
    public function findBy(array $where): array
    {
        $imageRepository = GeneralUtility::makeInstance(ImageRepository::class);
        $accordionRepository = GeneralUtility::makeInstance(AccordionRepository::class);
        $tabRepository = GeneralUtility::makeInstance(TabRepository::class);
        $carouselRepository = GeneralUtility::makeInstance(CarouselRepository::class);

        $joinedWhere = [];

        foreach ($where as $key => $value) {
            $joinedWhere[] = "$key = $value";
        }

        $qb = QueryBuilderFactory::create(self::TABLE_NAME);

        $result = $qb
            ->select('uid', 'header', 'subheader', 'bodytext')
            ->from(self::TABLE_NAME)
            ->where(implode(' AND ', $joinedWhere))
            ->andWhere('deleted = 0')
            ->addOrderBy('colPos')
            ->addOrderBy('sorting')
            ->execute()
            ->fetchAll();

        if (!is_array($result)) {
            throw new NotFoundException('No content found.');
        }

        foreach ($result as $key => $content) {
            $result[$key] += [
                'images' => $imageRepository->findByRelation(self::TABLE_NAME, $content['uid']),
                'accordion_items' => $accordionRepository->findByContentAndSourceLang($content['uid']),
                'tab_items' => $tabRepository->findByContentAndSourceLang($content['uid']),
                'carousel_items' => $carouselRepository->findByContentAndSourceLang($content['uid'])
            ];

            // images in carousel items
            foreach ($result[$key]['carousel_items'] as $itemKey => $carouselItem) {
                $result[$key]['carousel_items'][$itemKey]['images'] = $imageRepository->findByRelation(
                    CarouselRepository::TABLE_NAME,
                    $carouselItem['uid']
                );
            }

            // images in accordion items
            foreach ($result[$key]['accordion_items'] as $itemKey => $accordionItem) {
                $result[$key]['accordion_items'][$itemKey]['images'] = $imageRepository->findByRelation(
                    AccordionRepository::TABLE_NAME,
                    $accordionItem['uid']
                );
            }

            // images in tab items
            foreach ($result[$key]['tab_items'] as $itemKey => $tabItem) {
                $result[$key]['tab_items'][$itemKey]['images'] = $imageRepository->findByRelation(
                    TabRepository::TABLE_NAME,
                    $tabItem['uid']
                );
            }
        }

        return $result;
    }

    /**
     * Update single content.
     *
     * @param int $contentId
     * @param string $header
     * @param string $subheader
     * @param string $bodytext
     */
    public function update($contentId, $header, $subheader, $bodytext)
    {
        QueryBuilderFactory::create(self::TABLE_NAME)
            ->update(self::TABLE_NAME)
            ->where("uid = $contentId")
            ->set('header', $header)
            ->set('subheader', $subheader)
            ->set('bodytext', $bodytext)
            ->execute();
    }

    /**
     * Get already existing or create new target content.
     *
     * @param int $sourcePageId
     * @param int $sourceContentId
     * @param int $targetLangId
     * @return null|int
     * @throws NotFoundException
     */
    public function getOrCreateTargetContent($sourcePageId, $sourceContentId, $targetLangId)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);

        $targetContentId = $this->findContentIdByLang($sourcePageId, $sourceContentId, $targetLangId);

        if ($targetContentId === false) {
            $newContent = $queryBuilder
                ->select('*')
                ->from(self::TABLE_NAME)
                ->where("uid = $sourceContentId")
                ->andWhere('deleted = 0')
                ->execute()
                ->fetch();

            unset($newContent['uid']);
            $newContent['pid'] = $sourcePageId;
            $newContent['crdate'] = $newContent['tstamp'] = time();
            $newContent['sys_language_uid'] = $targetLangId;
            $newContent['l18n_parent'] = $newContent['l10n_source'] = $newContent['t3_origuid'] = $sourceContentId;

            $queryBuilder
                ->insert(self::TABLE_NAME)
                ->values($newContent)
                ->execute();

            $targetContentId = $this->findContentIdByLang($sourcePageId, $sourceContentId, $targetLangId);
        }

        if ($targetContentId === false) {
            throw new NotFoundException("Unable to create or find content '$sourceContentId' by language '$targetLangId'.");
        }

        return (int) $targetContentId;
    }

    /**
     * Find single content.
     *
     * @param int $sourcePageId
     * @param int $sourceContentId
     * @param int $targetLangId
     * @return bool|string
     */
    private function findContentIdByLang($sourcePageId, $sourceContentId, $targetLangId)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);

        $targetContentId = $queryBuilder
            ->select('uid')
            ->from(self::TABLE_NAME)
            ->where("sys_language_uid = $targetLangId")
            ->andWhere("pid = $sourcePageId")
            ->andWhere(
                $queryBuilder->expr()->eq(
                    'l18n_parent',
                    $queryBuilder->createNamedParameter($sourceContentId, \PDO::PARAM_INT)
                )
            )
            ->andWhere('deleted = 0')
            ->execute()
            ->fetchColumn();

        return $targetContentId;
    }
}
