<?php

namespace Memsource\MemsourceConnector\Domain\Model;

use Memsource\MemsourceConnector\Domain\Entity\Image;
use Memsource\MemsourceConnector\Exception\NotFoundException;

class ImageRepository extends BaseRepository
{
    const TABLE_NAME = 'sys_file_reference';

    /**
     * Get images for given content type.
     *
     * @param string $refTable
     * @param int $refUid
     * @return Image[]
     */
    public function findByRelation(string $refTable, $refUid): array
    {
        $result = QueryBuilderFactory::create(self::TABLE_NAME)
            ->select('ref.uid')
            ->addSelectLiteral('IF(ref.title IS NULL, meta.title, ref.title) AS title')
            ->addSelectLiteral('IF(ref.alternative IS NULL, meta.alternative, ref.alternative) AS alternative')
            ->addSelectLiteral('IF(ref.description IS NULL, meta.description, ref.description) AS description')
            ->from(self::TABLE_NAME, 'ref')
            ->leftJoin('ref', 'sys_file_metadata', 'meta', 'ref.uid_local = meta.file')
            ->where('ref.uid_foreign = ' . (int) $refUid)
            ->andWhere('ref.tablenames = "' . $refTable . '"')
            ->andWhere('ref.sys_language_uid = 0')
            ->execute()
            ->fetchAll();

        $images = [];

        foreach ($result as $item) {
            $images[] = new Image($item);
        }

        return $images;
    }

    /**
     * Bulk update images.
     *
     * @param array $images
     * @param int $sourcePageId
     * @param int $targetId
     * @param int $targetLanguageId
     * @param string $relatedTable
     */
    public function bulkUpdate($images, $sourcePageId, $targetId, $targetLanguageId, $relatedTable)
    {
        if (is_array($images)) {
            foreach ($images as $imageId => $image) {
                $targetImageId = $this->getOrCreateTarget($sourcePageId, $targetId, $imageId, $targetLanguageId, $relatedTable);
                $this->update($targetImageId, $image['title'], $image['alternative'], $image['description']);
            }
        }
    }

    /**
     * Get or create image in target lang.
     *
     * @param int $sourcePageId
     * @param int $sourceForeignId
     * @param int $sourceImageId
     * @param int $targetLangId
     * @param string $refTable
     * @return int
     * @throws NotFoundException
     */
    public function getOrCreateTarget($sourcePageId, $sourceForeignId, $sourceImageId, $targetLangId, $refTable)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);

        $targetId = $this->findIdByLangAndRef($sourcePageId, $sourceForeignId, $sourceImageId, $targetLangId, $refTable);

        if ($targetId === false) {
            $newImage = $this->findOneById($sourceImageId);

            unset($newImage['uid']);
            $newImage['pid'] = $sourcePageId;
            $newImage['uid_foreign'] = $sourceForeignId;
            $newImage['crdate'] = $newImage['tstamp'] = time();
            $newImage['sys_language_uid'] = $targetLangId;
            $newImage['tablenames'] = $refTable;
            $newImage['l10n_parent'] = $sourceImageId;

            $queryBuilder
                ->insert(self::TABLE_NAME)
                ->values($newImage)
                ->execute();

            $targetId = $this->findIdByLangAndRef($sourcePageId, $sourceForeignId, $sourceImageId, $targetLangId, $refTable);
        }

        if ($targetId === false) {
            throw new NotFoundException("Unable to create or find image '$sourceImageId' by language '$targetLangId'.");
        }

        return (int) $targetId;
    }

    /**
     * Store translation into target image.
     *
     * @param int $id
     * @param string $title
     * @param string $alternative
     * @param string $description
     */
    public function update($id, $title, $alternative, $description)
    {
        QueryBuilderFactory::create(self::TABLE_NAME)
            ->update(self::TABLE_NAME)
            ->where("uid = $id")
            ->set('title', $title)
            ->set('alternative', $alternative)
            ->set('description', $description)
            ->execute();
    }

    /**
     * Find id fo given target lang and reference type.
     *
     * @param int $sourcePageId
     * @param int $sourceContentId
     * @param int $sourceImageId
     * @param int $langId
     * @param string $refTable
     * @return bool|string
     */
    private function findIdByLangAndRef($sourcePageId, $sourceContentId, $sourceImageId, $langId, $refTable)
    {
        return QueryBuilderFactory::create(self::TABLE_NAME)
            ->select('uid')
            ->from(self::TABLE_NAME)
            ->where('sys_language_uid = ' . (int) $langId)
            ->andWhere('pid = ' . (int) $sourcePageId)
            ->andWhere('uid_foreign = ' . (int) $sourceContentId)
            ->andWhere('tablenames = "' . $refTable . '"')
            ->andWhere('l10n_parent = ' . (int) $sourceImageId)
            ->andWhere('deleted = 0')
            ->execute()
            ->fetchColumn();
    }
}
