<?php

namespace Memsource\MemsourceConnector\Domain\Model;

use Memsource\MemsourceConnector\Domain\Entity\Page;
use Memsource\MemsourceConnector\Exception\NotFoundException;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;

class NewsRepository extends BaseRepository
{
    const TABLE_NAME = 'tx_news_domain_model_news';
    const TOP_PARENT_ID = 1;
    const ID_PREFIX = 'news_';

    /**
     * Find news filtered by the parent one.
     *
     * @param int $parent
     * @return array
     */
    public function findByParent($parent): array
    {
        if (!$this->checkIfTableExists(self::TABLE_NAME)) {
            return [];
        }

        $query = QueryBuilderFactory::create(self::TABLE_NAME)
            ->select(
                'n.uid AS id',
                'n.title AS title',
                'n.tstamp AS modified'
            )
            ->addSelectLiteral(
                '0 AS directory'
            )
            ->from(self::TABLE_NAME, 'n');

        $query->where('n.pid = ' . max((int) $parent, self::TOP_PARENT_ID));

        $query->andWhere('n.sys_language_uid = 0')
            ->andWhere('n.l10n_parent = 0')
            ->andWhere('n.deleted = 0')
            ->addOrderBy('n.sorting');

        $news = $query->execute()->fetchAll();

        foreach ($news as $key => $item) {
            $news[$key]['size'] = $this->getNewsSizeWithContent($item['id']);
            $news[$key]['id'] = self::ID_PREFIX . $item['id'];
        }

        return $news;
    }

    /**
     * Check if given table exists
     *
     * @param string $table
     * @return bool
     */
    private function checkIfTableExists($table): bool
    {
        $tableColumns = GeneralUtility::makeInstance(ConnectionPool::class)
            ->getConnectionForTable($table)
            ->getSchemaManager()
            ->listTableColumns($table);

        return !(empty($tableColumns) === true);
    }

    /**
     * Calculate size of source news including content.
     *
     * @param int $newsUid
     * @return int
     */
    private function getNewsSizeWithContent($newsUid): int
    {
        $contentRepository = GeneralUtility::makeInstance(ContentRepository::class);

        try {
            $news = $this->findOneByUid($newsUid);
            $size = strlen(json_encode($news));
        } catch (NotFoundException $e) {
            $size = 0;
        }

        $content = $contentRepository->findBy(['tx_news_related_news' => $newsUid, 'sys_language_uid' => 0]);

        $size += strlen(json_encode($content));
        $size -= 2;
        $size = max(0, $size);

        return $size;
    }

    /**
     * Find single news by primary key.
     *
     * @param int $uid
     * @return Page
     * @throws NotFoundException
     */
    public function findOneByUid($uid): Page
    {
        $result = QueryBuilderFactory::create(self::TABLE_NAME)
            ->select('title', 'teaser', 'bodytext', 'alternative_title', 'keywords', 'description', 'sys_language_uid', 'l10n_parent')
            ->from(self::TABLE_NAME)
            ->where('uid = ' . (int) $uid)
            ->andWhere('deleted = 0')
            ->execute()
            ->fetch();

        if ($result === false || empty($result)) {
            throw new NotFoundException("News with uid '$uid' not found.");
        }

        return new Page($result);
    }

    /**
     * Update single page.
     *
     * @param int $pageId
     * @param string $title
     * @param array $page
     */
    public function update($pageId, $title, $page)
    {
        $qb = QueryBuilderFactory::create(self::TABLE_NAME);
        $qb->update(self::TABLE_NAME)
            ->where("uid = $pageId")
            ->set('title', $title)
            ->set('teaser', $page['teaser'])
            ->set('bodytext', $page['bodytext'])
            ->set('alternative_title', $page['alternativeTitle'])
            ->set('keywords', $page['keywords'])
            ->set('description', $page['description']);

        $qb->execute();
    }

    /**
     * Get already existing or create new target.
     *
     * @param int $sourceUid
     * @param int $targetLangId
     * @return null|int
     */
    public function getOrCreateTargetPage(int $sourceUid, int $targetLangId)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);
        $targetUid = $this->findUidByLang($sourceUid, $targetLangId);

        if ($targetUid === false) {
            $newRecord = $queryBuilder
                ->select('*')
                ->from(self::TABLE_NAME)
                ->where("uid = $sourceUid")
                ->andWhere('deleted = 0')
                ->execute()
                ->fetch();

            unset($newRecord['uid']);

            $newRecord['crdate'] = $newRecord['tstamp'] = time();
            $newRecord['sys_language_uid'] = $targetLangId;
            $newRecord['l10n_parent'] = $newRecord['l10n_source'] = $sourceUid;
            $newRecord['l10n_state'] = $newRecord['l10n_diffsource'] = '';

            $queryBuilder
                ->insert(self::TABLE_NAME)
                ->values($newRecord)
                ->execute();

            $targetUid = $this->findUidByLang($sourceUid, $targetLangId);
        }

        if ($targetUid === false) {
            throw new NotFoundException("Unable to find or create news with UID '$sourceUid' and language '$targetLangId'.");
        }

        return (int) $targetUid;
    }

    /**
     * Find ID of news by target lang.
     *
     * @param int $sourceUid
     * @param int $targetLangId
     * @return bool|string
     */
    private function findUidByLang($sourceUid, $targetLangId)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);

        $query = $queryBuilder
            ->select('uid')
            ->from(self::TABLE_NAME)
            ->where("sys_language_uid = $targetLangId")
            ->andWhere('deleted = 0');

        $query->andWhere(
            $queryBuilder->expr()->eq(
                'l10n_parent',
                $queryBuilder->createNamedParameter($sourceUid, \PDO::PARAM_INT)
            )
        );

        return $query->execute()->fetchColumn();
    }
}
