<?php

namespace Memsource\MemsourceConnector\Domain\Model;

use Memsource\MemsourceConnector\Domain\Entity\Page;
use Memsource\MemsourceConnector\Domain\Utility\Db;
use Memsource\MemsourceConnector\Domain\Utility\Typo3;
use Memsource\MemsourceConnector\Exception\NotFoundException;
use TYPO3\CMS\Core\Utility\GeneralUtility;

class PageRepository
{
    const TABLE_NAME = 'pages';
    const TABLE_NAME_TRANSLATIONS = 'pages_language_overlay';

    const TOP_PARENT_ID = 1;

    const EXT_FIELDS = [
        'seoTitle' => 'seo_title',
        'ogTitle' => 'og_title',
        'ogDescription' => 'og_description',
        'twitterTitle' => 'twitter_title',
        'twitterDescription' => 'twitter_description',
    ];

    /**
     * Get name of table used for translations.
     *
     * @return string
     */
    public static function getTableName(): string
    {
        return (Typo3::getMajorVersion() < 9) ? self::TABLE_NAME_TRANSLATIONS : self::TABLE_NAME;
    }

    /**
     * Find pages filtered by parent one.
     *
     * @param int|null $parent
     * @return array
     */
    public function findByParent($parent): array
    {
        $query = QueryBuilderFactory::create(self::TABLE_NAME)
            ->select(
                'p.uid AS id',
                'p.title AS title',
                'p.tstamp AS modified'
            )
            ->addSelectLiteral(
                'IF(COUNT(s.pid) > 0 OR p.doktype = 254, 1, 0) AS directory'
            )
            ->from(self::TABLE_NAME, 'p')
            ->leftJoin('p', self::TABLE_NAME, 's', 's.pid = p.uid');

        if ($parent === null) {
            $query->where('p.pid = 0');
        } else {
            $query->where('p.pid = ' . max((int) $parent, self::TOP_PARENT_ID));
        }

        if (Typo3::getMajorVersion() > 8) {
            $query->andWhere('p.sys_language_uid = 0')
                ->andWhere('p.l10n_parent = 0');
        }

        $query
            ->andWhere('(p.doktype IN (1, 199, 254)) OR (p.doktype = 4 AND s.pid IS NOT NULL)')
            ->andWhere('p.deleted = 0')
            ->groupBy('p.uid')
            ->orderBy('p.pid', ' != 0')
            ->addOrderBy('p.sorting');

        $pages = $query->execute()->fetchAll();
        $lastIndex = count($pages);

        foreach ($pages as $key => $page) {
            $pages[$key]['id'] = (string) $page['id'];
            $pages[$key]['size'] = $this->getPageSizeWithContent($page['id']);

            if ($page['directory'] === 1) {
                $pages[$lastIndex] = $pages[$key];
                $pages[$lastIndex]['directory'] = 0;
                $lastIndex++;
            }
        }

        return $pages;
    }

    /**
     * Find single page by primary key.
     *
     * @param int $uid
     * @param int|null $languageId
     * @return Page
     * @throws NotFoundException
     */
    public function findOneByUid($uid, $languageId = null): Page
    {
        $imageRepository = GeneralUtility::makeInstance(ImageRepository::class);

        if ($languageId && (Typo3::getMajorVersion() < 9)) {
            $result = QueryBuilderFactory::create(self::TABLE_NAME_TRANSLATIONS)
                ->select('title', 'nav_title', 'subtitle', 'abstract', 'keywords', 'description', 'sys_language_uid')
                ->addSelectLiteral('pid AS l10n_parent', 'NULL AS og_title', 'NULL AS seo_title', 'NULL AS og_description', 'NULL AS twitter_title', 'NULL AS twitter_description')
                ->from(self::TABLE_NAME_TRANSLATIONS)
                ->where('uid = ' . (int) $uid)
                ->andWhere('doktype = 1')
                ->andWhere('deleted = 0')
                ->execute()
                ->fetch();
        } else {
            $query = QueryBuilderFactory::create(self::TABLE_NAME)
                ->select('title', 'nav_title', 'subtitle', 'abstract', 'keywords', 'description');

            $columnsInDb = array_flip(Db::getColumnNames(self::TABLE_NAME));

            foreach (self::EXT_FIELDS as $column) {
                if (isset($columnsInDb[$column])) {
                    $query->addSelect($column);
                } else {
                    $query->addSelectLiteral("NULL AS $column");
                }
            }

            if (Typo3::getMajorVersion() < 9) {
                $query->addSelectLiteral('0 AS sys_language_uid', '0 AS l10n_parent');
            } else {
                $query->addSelect('sys_language_uid', 'l10n_parent');
            }

            $result = $query->from(self::TABLE_NAME)
                ->where('uid = ' . (int) $uid)
                ->andWhere('deleted = 0')
                ->execute()
                ->fetch();
        }

        if ($result === false || empty($result)) {
            throw new NotFoundException("Page with uid '$uid' not found.");
        }

        $result['images'] = $imageRepository->findByRelation(self::TABLE_NAME, $uid);

        return new Page($result);
    }

    /**
     * Update single page.
     *
     * @param int $pageId
     * @param string $title
     * @param array $page
     */
    public function update($pageId, $title, $page)
    {
        $qb = QueryBuilderFactory::create(self::TABLE_NAME);
        $qb->update((Typo3::getMajorVersion() < 9) ? self::TABLE_NAME_TRANSLATIONS : self::TABLE_NAME)
            ->where("uid = $pageId")
            ->set('title', $title)
            ->set('nav_title', $page['navTitle'])
            ->set('subtitle', $page['subtitle'])
            ->set('abstract', $page['abstract'])
            ->set('keywords', $page['keywords'])
            ->set('description', $page['description']);

        foreach (self::EXT_FIELDS as $columnJson => $columnDb) {
            if (isset($columnsInDb[$columnDb], $page[$columnJson])) {
                $qb->set($columnDb, $page[$columnJson]);
            }
        }

        $qb->execute();
    }

    /**
     * Get already existing or create new target page.
     *
     * @param int $sourcePageId
     * @param int $targetLangId
     * @return null|int
     */
    public function getOrCreateTargetPage(int $sourcePageId, int $targetLangId)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);
        $translationQueryBuilder = QueryBuilderFactory::create(self::TABLE_NAME_TRANSLATIONS);

        $targetPageId = $this->findPageIdByLang($sourcePageId, $targetLangId);

        if ($targetPageId === false) {
            $newPage = $queryBuilder
                ->select('*')
                ->from(self::TABLE_NAME)
                ->where("uid = $sourcePageId")
                ->andWhere('deleted = 0')
                ->execute()
                ->fetch();

            unset($newPage['uid']);

            $newPage['crdate'] = $newPage['tstamp'] = time();
            $newPage['sys_language_uid'] = $targetLangId;
            $newPage['l10n_parent'] = $newPage['l10n_source'] = $sourcePageId;
            $newPage['l10n_diffsource'] = '';
            $newPage['tx_impexp_origuid'] = 0;

            if (Typo3::getMajorVersion() < 10) {
                $newPage['l10n_state'] = '{"starttime":"parent","endtime":"parent","url":"parent",' .
                    '"lastUpdated":"parent","newUntil":"parent","no_search":"parent","shortcut":"parent",' .
                    '"shortcut_mode":"parent","author":"parent","author_email":"parent","media":"parent",' .
                    '"og_image":"parent","twitter_image":"parent"}';
            } else {
                $newPage['l10n_state'] = '{"starttime":"parent","endtime":"parent","nav_hide":"parent","url":"parent",' .
                    '"lastUpdated":"parent","newUntil":"parent","no_search":"parent","shortcut":"parent",' .
                    '"shortcut_mode":"parent","content_from_pid":"parent","author":"parent","author_email":"parent",' .
                    '"media":"parent","og_image":"parent","twitter_image":"parent"}';
            }

            if (Typo3::getMajorVersion() < 9) {
                $newPage['pid'] = $sourcePageId;

                $newPageTranslation = [];

                foreach (Db::getColumnNames(self::TABLE_NAME_TRANSLATIONS) as $columnName) {
                    if (isset($newPage[$columnName])) {
                        $newPageTranslation[$columnName] = $newPage[$columnName];
                    }
                }

                $translationQueryBuilder
                    ->insert(self::TABLE_NAME_TRANSLATIONS)
                    ->values($newPageTranslation)
                    ->execute();
            } else {
                $queryBuilder
                    ->insert(self::TABLE_NAME)
                    ->values($newPage)
                    ->execute();
            }

            $targetPageId = $this->findPageIdByLang($sourcePageId, $targetLangId);
        }

        if ($targetPageId === false) {
            throw new NotFoundException("Unable to find or create page '$sourcePageId' with language '$targetLangId'.");
        }

        return (int) $targetPageId;
    }

    /**
     * Calculate size of source page including content.
     *
     * @param int $pageUid
     * @return int
     */
    private function getPageSizeWithContent($pageUid): int
    {
        $contentRepository = GeneralUtility::makeInstance(ContentRepository::class);

        try {
            $page = $this->findOneByUid($pageUid);
            $size = strlen(json_encode($page));
        } catch (NotFoundException $e) {
            $size = 0;
        }

        $content = $contentRepository->findBy(['pid' => $pageUid, 'sys_language_uid' => 0]);

        $size += strlen(json_encode($content));
        $size -= 2;
        $size = max(0, $size);

        return $size;
    }

    /**
     * Find ID of page by target lang.
     *
     * @param int $sourcePageId
     * @param int $targetLangId
     * @return bool|string
     */
    private function findPageIdByLang($sourcePageId, $targetLangId)
    {
        $queryBuilder = QueryBuilderFactory::create(self::TABLE_NAME);

        $query = $queryBuilder
            ->select('uid')
            ->from((Typo3::getMajorVersion() < 9) ? self::TABLE_NAME_TRANSLATIONS : self::TABLE_NAME)
            ->where("sys_language_uid = $targetLangId")
            ->andWhere('deleted = 0');

        $query->andWhere(
            $queryBuilder->expr()->eq(
                (Typo3::getMajorVersion() < 9) ? 'pid' : 'l10n_parent',
                $queryBuilder->createNamedParameter($sourcePageId, \PDO::PARAM_INT)
            )
        );

        return $query->execute()->fetchColumn();
    }
}
