<?php

namespace Memsource\MemsourceConnector\Domain\Service;

use TYPO3\CMS\Core\Core\Environment;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Mail\MailMessage;

class Logger
{
	const LOG_FILE = 'memsource_connector.typo3.log';
	const LOG_FILE_RECIPIENT = 'integrations@phrase.com';

    /**
     * Log HTTP request.
     *
     * @param string $method
     * @param string $url
     * @param array $post
     * @param string $code
     * @param array $response
     * @param null|\Exception $exception
     * @throws \Exception
     */
	public function logHttpRequest($method, $url, $post, $code, $response, $exception = null)
	{
		$message = "HTTP-REQUEST\n";
		$message .= "\nREQUEST:\n";
		$message .= "$method $url\n";

		if ($method === 'POST') {
            foreach ($post as $key => $value) {
                $message .= " - $key: $value\n";
            }
		}

		if ($exception !== null) {
			$message .= "\nEXCEPTION:\nThrown $exception\n";
		}

		$message .= "\nRESPONSE:\nHTTP $code\n";
        $message .= json_encode($response, JSON_PRETTY_PRINT) . "\n";
		$message .= "\n------------------\n";

		$this->log($message);
	}

	/**
	 * Get size of logs in bytes.
	 *
	 * @return int
	 */
	public function getLogFileSize(): int
    {
		$logFile = $this->getLogFileNameWithPath();

		if (file_exists($logFile))
		{
			return filesize($logFile) ?: 0;
		}

		return 0;
	}

	/**
	 * Force download log file in web browser.
	 */
	public function forceDownloadLogFile()
	{
		$this->logSystemInfo();
		header("Content-Type: text/plain\n");
		header('Content-Disposition: attachment; filename="' . $this->getLogFileNameForDownload() . '"\n');
		readfile($this->getLogFileNameWithPath());
		exit();
	}

	/**
	 * Send logs to Phrase.
     *
     * @param string $baseUri Server URL
     *
     * @return int Result
	 */
	public function sendLogFileToEmail($baseUri): int
    {
		$this->logSystemInfo();

        if (isset($GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'])) {
            $senderName = $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'];
        } else {
            $senderName = "TYPO3 - $baseUri";
        }

        $url = parse_url($baseUri);

        if (isset($url['host'])) {
            $senderEmail = 'noreply@' . $url['host'];
        } else {
            $senderEmail = 'noreply@typo3.org';
        }

        $mail = GeneralUtility::makeInstance(MailMessage::class);
        $mail->setFrom([$senderEmail => $senderName]);
        $mail->setTo([self::LOG_FILE_RECIPIENT => 'Phrase']);
        $mail->setSubject("TYPO3 logs from $baseUri");
        $mail->setBody("TYPO3 logs from $baseUri");
        $mail->attach(
            \Swift_Attachment::fromPath($this->getLogFileNameWithPath())->setFilename($this->getLogFileNameForDownload())
        );

        return $mail->send();
	}

	/**
	 * Delete logs.
	 */
	public function emptyLogFile()
	{
        file_put_contents($this->getLogFileNameWithPath(), '', LOCK_EX);
	}

	/**
	 * Log Typo3, connector and PHP versions.
	 */
	public function logSystemInfo()
	{
        $message = 'TYPO3 version: ';

	    if (defined('TYPO3_version')) {
	        $message .= TYPO3_version;
        } else {
	        $message .= 'unknown';
        }

	    $message .= ', Phrase Connector version: ';

	    $ext = 'memsource_connector';
	    $conf = 'ext_emconf.php';

	    if (defined('PATH_typo3conf') && file_exists(PATH_typo3conf . "ext/$ext/$conf")) {
	        include PATH_typo3conf . "ext/$ext/$conf";
        } elseif (file_exists(__DIR__ . "/../../../$conf")) {
	        include __DIR__ . "/../../../$conf";
        }

        $message .= current($EM_CONF)['version'] ?? 'unknown';
	    $message .= ', PHP version: ' . PHP_VERSION;

	    $this->log($message);
	}

	/**
	 * Add a new entry to the log file.
	 *
	 * @param string $message Log entry.
	 */
	public function log($message)
	{
	    $logFile = $this->getLogFileNameWithPath();
	    $logMessage = date('c') . " - $message\n";

	    if (file_put_contents($logFile, $logMessage, LOCK_EX | FILE_APPEND) === false) {
            throw new \Exception("Unable to write to the log file '$logFile'.");
        }
	}

	/**
	 * Get log file with absolute path.
	 *
	 * @return  string
	 */
	private function getLogFileNameWithPath(): string
    {
        if (defined('PATH_site')) {
            $path = PATH_site;
        } else {
            $path = Environment::getPublicPath();
        }

        if (substr($path, -1) !== '/') {
            $path .= '/';
        }

        if (file_exists($path . 'typo3temp/var/log')) {
            return $path . 'typo3temp/var/log/' . self::LOG_FILE;
        }

        return $path . 'typo3temp/var/logs/' . self::LOG_FILE;
	}

	/**
	 * Get name of downloaded file.
	 *
	 * @return  string
	 */
	private function getLogFileNameForDownload(): string
    {
		return date('Y-m-d_H-i-s') . '_' . self::LOG_FILE;
	}
}
